/**
 * Copyright 2013-2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @providesModule LinkedStateMixin
 * @typechecks static-only
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _ReactLink = require('./ReactLink');

var _ReactLink2 = _interopRequireDefault(_ReactLink);

var _ReactStateSetters = require('./ReactStateSetters');

var _ReactStateSetters2 = _interopRequireDefault(_ReactStateSetters);

/**
 * A simple mixin around ReactLink.forState().
 */
var LinkedStateMixin = {
  /**
   * Create a ReactLink that's linked to part of this component's state. The
   * ReactLink will have the current value of this.state[key] and will call
   * setState() when a change is requested.
   *
   * @param {string} key state key to update. Note: you may want to use keyOf()
   * if you're using Google Closure Compiler advanced mode.
   * @return {ReactLink} ReactLink instance linking to the state.
   */
  linkState: function linkState(key) {
    return new _ReactLink2['default'](this.state[key], _ReactStateSetters2['default'].createStateKeySetter(this, key));
  }
};

exports['default'] = LinkedStateMixin;
module.exports = exports['default'];