/**
 * Copyright 2013-2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @typechecks
 * @providesModule ReactCSSTransitionGroupChild
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _srcIndex = require('../src/index');

var _srcIndex2 = _interopRequireDefault(_srcIndex);

var _srcChildren = require('../src/Children');

var Children = _interopRequireWildcard(_srcChildren);

var _srcReactDOM = require('../src/ReactDOM');

var ReactDOM = _interopRequireWildcard(_srcReactDOM);

var _utilsCSSCore = require('./utils/CSSCore');

var _utilsCSSCore2 = _interopRequireDefault(_utilsCSSCore);

var _ReactTransitionEvents = require('./ReactTransitionEvents');

var _ReactTransitionEvents2 = _interopRequireDefault(_ReactTransitionEvents);

var onlyChild = Children.only;

// We don't remove the element from the DOM until we receive an animationend or
// transitionend event. If the user screws up and forgets to add an animation
// their node will be stuck in the DOM forever, so we detect if an animation
// does not start and if it doesn't, we just call the end listener immediately.
var TICK = 17;

var ReactCSSTransitionGroupChild = _srcIndex2['default'].createClass({
  displayName: 'ReactCSSTransitionGroupChild',

  propTypes: {
    name: _srcIndex2['default'].PropTypes.oneOfType([_srcIndex2['default'].PropTypes.string, _srcIndex2['default'].PropTypes.shape({
      enter: _srcIndex2['default'].PropTypes.string,
      leave: _srcIndex2['default'].PropTypes.string,
      active: _srcIndex2['default'].PropTypes.string
    }), _srcIndex2['default'].PropTypes.shape({
      enter: _srcIndex2['default'].PropTypes.string,
      enterActive: _srcIndex2['default'].PropTypes.string,
      leave: _srcIndex2['default'].PropTypes.string,
      leaveActive: _srcIndex2['default'].PropTypes.string,
      appear: _srcIndex2['default'].PropTypes.string,
      appearActive: _srcIndex2['default'].PropTypes.string
    })]).isRequired,

    // Once we require timeouts to be specified, we can remove the
    // boolean flags (appear etc.) and just accept a number
    // or a bool for the timeout flags (appearTimeout etc.)
    appear: _srcIndex2['default'].PropTypes.bool,
    enter: _srcIndex2['default'].PropTypes.bool,
    leave: _srcIndex2['default'].PropTypes.bool,
    appearTimeout: _srcIndex2['default'].PropTypes.number,
    enterTimeout: _srcIndex2['default'].PropTypes.number,
    leaveTimeout: _srcIndex2['default'].PropTypes.number
  },

  transition: function transition(animationType, finishCallback, userSpecifiedDelay) {
    var node = ReactDOM.findDOMNode(this);

    if (!node) {
      if (finishCallback) {
        finishCallback();
      }
      return;
    }

    var className = this.props.name[animationType] || this.props.name + '-' + animationType;
    var activeClassName = this.props.name[animationType + 'Active'] || className + '-active';
    var timeout = null;

    var endListener = function endListener(e) {
      if (e && e.target !== node) {
        return;
      }

      clearTimeout(timeout);

      _utilsCSSCore2['default'].removeClass(node, className);
      _utilsCSSCore2['default'].removeClass(node, activeClassName);

      _ReactTransitionEvents2['default'].removeEndEventListener(node, endListener);

      // Usually this optional callback is used for informing an owner of
      // a leave animation and telling it to remove the child.
      if (finishCallback) {
        finishCallback();
      }
    };

    _utilsCSSCore2['default'].addClass(node, className);

    // Need to do this to actually trigger a transition.
    this.queueClass(activeClassName);

    // If the user specified a timeout delay.
    if (userSpecifiedDelay) {
      // Clean-up the animation after the specified delay
      timeout = setTimeout(endListener, userSpecifiedDelay);
      this.transitionTimeouts.push(timeout);
    } else {
      // DEPRECATED: this listener will be removed in a future version of react
      _ReactTransitionEvents2['default'].addEndEventListener(node, endListener);
    }
  },

  queueClass: function queueClass(className) {
    this.classNameQueue.push(className);

    if (!this.timeout) {
      this.timeout = setTimeout(this.flushClassNameQueue, TICK);
    }
  },

  flushClassNameQueue: function flushClassNameQueue() {
    if (this.isMounted()) {
      this.classNameQueue.forEach(_utilsCSSCore2['default'].addClass.bind(_utilsCSSCore2['default'], ReactDOM.findDOMNode(this)));
    }
    this.classNameQueue.length = 0;
    this.timeout = null;
  },

  componentWillMount: function componentWillMount() {
    this.classNameQueue = [];
    this.transitionTimeouts = [];
  },

  componentWillUnmount: function componentWillUnmount() {
    if (this.timeout) {
      clearTimeout(this.timeout);
    }
    this.transitionTimeouts.forEach(function (timeout) {
      clearTimeout(timeout);
    });
  },

  componentWillAppear: function componentWillAppear(done) {
    if (this.props.appear) {
      this.transition('appear', done, this.props.appearTimeout);
    } else {
      done();
    }
  },

  componentWillEnter: function componentWillEnter(done) {
    if (this.props.enter) {
      this.transition('enter', done, this.props.enterTimeout);
    } else {
      done();
    }
  },

  componentWillLeave: function componentWillLeave(done) {
    if (this.props.leave) {
      this.transition('leave', done, this.props.leaveTimeout);
    } else {
      done();
    }
  },

  render: function render() {
    return onlyChild(this.props.children);
  }
});

exports['default'] = ReactCSSTransitionGroupChild;
module.exports = exports['default'];